% polaire  Sert  afficher les fiagrammes polaires des diffrents blocs.
% 
% Syntaxe
%     fig=polaire(data, blocs,titre)
%       fig    : matrice des figures
%       data   : matrices des donnes que l'on doit plotter
%                  le nombre de colonnes est le nombre de points
%                  plotts par courbe
%                  le nombre de lignes est le nombre de courbes, si
%                  aucun argument n'est pass dans 'blocs'
%       blocs  : indices de data  plotter.
%	titre  : titre des fractures	
%
% Prerequis de programmation
% @Scripts employs
% polairefig.m, figure prpare avec 6 axes pour les diagrammes polaires 
%    et une fentre d'dition pour le titre et le numro de page
%	Toute ressemblance avec imaging.m n'est pas fortuite : ils posent les
% memes problmes, qui ont t rsolus en mme temps.
%
%	@ donnes employes
%  Chaque figure possde des proprits spcifiques
%	- son champ 'Userdata' est occup par un identificateur identique
%		 toutes les figures 'diagrammes polaires'
%	- des donnes associes par setappdata :
%		- 'id' : mme id plus haut
%		- 'pages' : liste des figures des diagrammes polaires
%		- numero : numro de la page parmi ces figures
%
% Aide modifie le : 07/01/2000
%

function fig=polaire(data, blocs,profdata,titre)

% si data est une chaine de caractre, on appelle en fait un
% callbak de la figure polairefig   (un callback est un script
% appel par un lment d'interface graphique) 

if ~ischar(data)
  % Fonctionnement normal
  if nargin<2
    blocs=1:size(data,1);
  else 
    if size(blocs,1)>size(blocs,2)
      blocs=blocs';
    end
  end
  %%%%% Preparation des donnes
  % le nombre de colonnes de data est le nombre de points plotts
  nombre=size(data,2);      
  theta=[0:2*pi/nombre:2*pi];
  % On referme la courbe du digagramme polaire en rptant le premier point 
  % pour chaque point
  data=[data data(:,1)];
  rho=data(blocs,:);
   
  %%%%%%% Plot des donnes
  % La figure est prpar par polairefig.m, script gnr  l'aide de 'guide'
  %                                 ('help guide', pour plus de dtails)
  nombreaxes=length(blocs);
  nombrepages=ceil(nombreaxes/6);
  id=now;       % on identifie la srie de figure par l'heure de lancement du
                % programme
  if nargin==4
    name=titre;
  else
    name='';
  end
   
  % trac des premires pages, sans obstacle majeur
  for i=1:(nombrepages-1)
    fig(i)=polairefig;
    set(gcf,'userdata',id);
    polaire('setmenu');      % Mise en place des menus
    set(findobj(gcf,'Tag','pagenumber'),'string',[num2str(i) '/' ...
		    num2str(nombrepages)]); 
    set(findobj(gcf,'Tag','pagetitle'),'string', name);
    if nargin>=3
      for j=1:6
	compteur=6*(i-1)+j;
	axes(findobj(gcf,'Type','axes','Tag',['ax' num2str(j)]));
	rhovu=rho(compteur,:);
	polar(theta, abs(rhovu));
	hold on;
	indicesmoins=find(rhovu<0);
	polar(theta(indicesmoins), abs(rhovu(indicesmoins)),'r.');
	view(-90,90);
	title({['bloc ' num2str(blocs(compteur))],...
	       [num2str(profdata(blocs(compteur))) ' m']});
      end
    else
      for j=1:6
	compteur=6*(i-1)+j;
	axes(findobj(gcf,'Type','axes','Tag',['ax' num2str(j)]));
	rhovu=rho(compteur,:);
	polar(theta, abs(rhovu));
	hold on;
	indicesmoins=find(rhovu<0);
	polar(theta(indicesmoins), abs(rhovu(indicesmoins)),'r.');
	view(-90,90);
	title(['bloc ' num2str(blocs(compteur))]);
      end
    end
  end
  % pour la dernire page, on plotte pas les axes inutiles
  fig(nombrepages)=polairefig;
  set(gcf,'userdata',id);
  polaire('setmenu');
  set(findobj(gcf,'Tag','pagenumber'),'string',[num2str(nombrepages) ...
		    '/' num2str(nombrepages)]); 
  set(findobj(gcf,'Tag','pagetitle'),'string', name);   
  
  % traitement derniere page
  lastaxnumber=mod(nombreaxes,6);
  if lastaxnumber==0
    lastaxnumber=6;
  end
  if nargin>=3
    for j=1:lastaxnumber
      compteur=6*(nombrepages-1)+j;
      axes(findobj(gcf,'Type','axes','Tag',['ax' num2str(j)]));
      rhovu=rho(compteur,:);
      polar(theta, abs(rhovu));
      hold on;
      indicesmoins=find(rhovu<0);
      polar(theta(indicesmoins), abs(rhovu(indicesmoins)),'r.');
      view(-90,90); % Rotation 0 pointe vers le haut
      title({['bloc ' num2str(blocs(compteur))], ...
	     [num2str(profdata(blocs(compteur))) ' m']},...
	    'Fontsize',6); 
    end
  else
    for j=1:lastaxnumber
      compteur=6*(nombrepages-1)+j;
      axes(findobj(gcf,'Type','axes','Tag',['ax' num2str(j)]));
      rhovu=rho(compteur,:);
      polar(theta, abs(rhovu));
      hold on;
      indicesmoins=find(rhovu<0);
      polar(theta(indicesmoins), abs(rhovu(indicesmoins)),'r.');
      view(-90,90);
      title(['bloc ' num2str(blocs(compteur))]);
    end
  end
  % On limine les axes en excs
  for j=(lastaxnumber+1):6
    delete(findobj(gcf,'Type','axes','Tag',['ax' num2str(j)]));
  end
  % On enregistre la liste des pages
  for i=1:length(fig)
    figure(fig(i))
    setappdata(gcf,'pages',fig); % 'saving', 'closing'
    setappdata(gcf,'numero',i);  % 'printing', 'saving'
    setappdata(gcf,'id',id);
  end
   
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%% CALLBACKS DE L'INTERFACE GRAPHIQUE %%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
else
  switch data
   case 'text'
    set(gcbo,'Style','edit','Callback','polaire(''edit'')');
   case 'edit'
    set(gcbo,'Style','text','Callback','polaire(text)');
    samefig=findobj('Userdata',get(gcbf,'Userdata'));
    name=get(gcbo,'string');
    figureappel=gcf;
    for i=1:length(samefig)                         
      set(findobj(figure(samefig(i)),'Tag','pagetitle'),'string', name);
    end
    figure(figureappel);
    % fermeture des fentres
   case 'setmenu'
    idpolar=get(gcbf,'Userdata');
    polarmenu=uimenu('Label','Figures',...
		     'foregroundcolor',[0.5 0.5 0]);
    printmenu=uimenu(polarmenu,'Label','Tout imprimer',...
		     'Callback','polaire(''printing'');');
    savemenu=uimenu(polarmenu,'Label','Tout sauvegarder',...
		    'Callback','polaire(''saving'');');
    closemenu=uimenu(polarmenu,'Label','Tout fermer',...
		     'Callback','polaire(''closing'');');
    firstmenu=uimenu('Label','|<<',...
		     'Userdata',idpolar,...
		     'Callback','polaire(''first'')',...
		     'foregroundcolor',[1 0 0]);
    previousmenu=uimenu('Label','<<',...
			'Userdata',idpolar,...
			'Callback','polaire(''previous'')',...
			'foregroundcolor',[1 0 0]);
    nextmenu=uimenu('Label','>>',...
		    'Userdata',idpolar,...
		    'Callback','polaire(''next'')',...
		    'foregroundcolor',[1 0 0]);
    lastmenu=uimenu('Label','>>|',...
		    'Userdata',idpolar,...
		    'Callback','polaire(''last'')',...
		    'foregroundcolor',[1 0 0]);
   case 'printing'
    idpolar=get(gcbf,'Userdata');
    figures=findobj('Type','figure','Userdata',idpolar);
    disp('L''impression de tous les diagrammes polaires commence')
    for i=1:length(figures);
      figure(figures(i));
      numero=getappdata(figures(i),'numero');
      disp(['impression de la page ' num2str(numero) ' des diagrammes']);
      print -dwinc;
    end
   case 'saving'
    formatlist=str2mat('jpeg','bmp','mfig','mmat','fig','tiff',...
		       'png','hdf','pcx','xwd');
    [format,ok]=listdlg('Name','Format ', ...
			'SelectionMode','single',...
			'Liststring',formatlist,...
			'ListSize',[100 140],...
			'fus',10);
    if ok
      format=formatlist(format,:);
    else
      return
    end
    name=inputdlg('Nom de sauvegarde :','Nom de sauvegarde');
    name=name{1};
    if isempty(name)
      return
    end
    % On obtient la liste des pages initialement cres.
    pages=getappdata(gcbf,'pages');
    % On vrifie que ces pages existent bien
    indices=find(ishandle(pages));
    pages=pages(indices);
    % On verifie aussi qu'elles corespondent bien  la diagraphie
    idpages=get(pages,'Userdata');
    % Idpages est de la structure cell si length(pages)>1
    id=getappdata(gcbf,'id');
    if iscell(idpages)
      indices=find([idpages{:}]==id);
    else
      indices=find(idpages==id);
    end
    pages=pages(indices);
    h=waitbar(0,'Sauvegarde des figures...');
    nombre=length(pages);
    for i=1:nombre;
      figure(pages(i));
      numero=getappdata(gcf,'numero');
      namesave=[name num2str(numero) '.' format];
      waitbar((i-1)/nombre,h);
      feval('saveas',gcf,namesave,format);
      waitbar(i/nombre,h);
      disp([ 'La page ' num2str(numero)...
	     ' de la diagraphie sauve est sous le nom ' namesave]);         
    end
    close(h);
    disp(['Figures sauvegardes dans le rpertoire courant ' pwd]);
   case 'closing'
    % Attention : Matlab tendait  faire des segmentation fault 
    % quand on fermait tout interactivement  cause d'un appel  findobj.
    % De plus quand Matlab a tourn suffisamment longtemps, son tas est
    % particulirement bordlique. Les chances de segfault sont augmentes.
    %
    % Voila pourquoi ce bout de code peut sembler particulirement tordu.
    %      id=get(gcf,'Userdata');
    id=getappdata(gcbf,'id');
    % pour trouver les pages, on utilise l'utilitaire set/getappdata
    % plutot que findobj
    % On obtient la liste des pages initialement cres.
    pages=getappdata(gcbf,'pages');
    for i=1:length(pages)
      % On vrifie qu'elles existent bien
      if ishandle(pages(i)) & ~isequal(gcbf,pages(i)) 
      % Qu'elles correspondent bien aux figures des diagrammes polaire
        try
	  idpage=getappdata(pages(i),'id');
          % Si oui, alors on ferme
	  if id==idpage
	    close(pages(i));
	  end
	catch
	end
      end
    end
    % On prpare la dernire figure,  se fermer
    clf;
    set(gcf,'ButtonDownFcn','closereq',...
	    'Name','Cliquez pour fermer',...
	    'Menubar','none');
    set(findobj(gcf,'Type','image'),'ButtonDownFcn','closereq');
    set(findobj(gcf,'Type','text'),'ButtonDownFcn','closereq');
   case 'close'
    closereq;
   case 'first'
    pages=getappdata(gcbf,'pages');
    % On ne retient que les pages existantes ...
    indices=find(ishandle(pages));
    pages=pages(indices);
    % ... avec l'identificateur de la diagraphie
    id=get(gcbf,'Userdata');
    idpages=get(pages,'Userdata');
    if iscell(idpages)
      indices=find([idpages{:}]==id);
    else
      indices=find(idpages==id);
    end
    pages=pages(indices);
    % Aller 1e page
    figure(pages(length(1)));
   case 'previous'
    pages=getappdata(gcbf,'pages');
    numeroini=1:length(pages);
    % On ne retient que les pages existantes ...
    indices=find(ishandle(pages));
    pages=pages(indices);
    numeroreste=numeroini(indices);
    % ... avec l'identificateur de la diagraphie
    id=get(gcbf,'Userdata');
    idpages=get(pages,'Userdata');
    if iscell(idpages)
      indices=find([idpages{:}]==id);
    else
      indices=find(idpages==id);
    end
    pages=pages(indices);
    numeroreste=numeroreste(indices);
    % Aller page suivante
    numero=getappdata(gcbf,'numero');
    numero=find(numeroreste==numero);
    if numero~=1
      figure(pages(numero-1));
    end
   case 'next'
    pages=getappdata(gcbf,'pages');
    numeroini=1:length(pages);
    % On ne retient que les pages existantes ...
    indices=find(ishandle(pages));
    pages=pages(indices);
    numeroreste=numeroini(indices);
    % ... avec l'identificateur de la diagraphie
    id=get(gcbf,'Userdata');
    idpages=get(pages,'Userdata');
    if iscell(idpages)
      indices=find([idpages{:}]==id);
    else
      indices=find(idpages==id);
    end
    pages=pages(indices);
    numeroreste=numeroreste(indices);
    % Aller page suivante
    numero=getappdata(gcbf,'numero');
    numero=find(numeroreste==numero);
    if numero~=length(pages)
      figure(pages(numero+1));
    end
   case 'last'
    % Parmi tous les pages cres,
    pages=getappdata(gcbf,'pages');
    % On ne retient que les pages existantes ...
    indices=find(ishandle(pages));
    pages=pages(indices);
    % ... et avec l'identificateur de la diagraphie
    id=get(gcbf,'Userdata');
    idpages=get(pages,'Userdata');
    if iscell(idpages)
      indices=find([idpages{:}]==id);
    else
      indices=find(idpages==id);
    end
    pages=pages(indices);
    % Aller 1e page
    figure(pages(length(pages)));   
  end
end

