% sinusoide
%
% Ajustage manuel d'une sinusoide sur une fracture.
%
%  sinusoide(donnee,PROF,V,orientation)
%           donnee : imagerie electrique.
%                    On rappelle que ce programme n'a de sens que
%                   si l'imagerie a ete prealablement ORIENTEE !
%           PROF   : donnee des profondeurs
%           V      : [V1 V2]  : V1 : valeur de blocs debut
%                               V2 : valeur de blocs fin
%           orientation : [lambda gamma] : 
%                               lambda : incidence en degres
%                               gamma  : azimuth en degres
%
%  SYNTAXE
%       sinusoide(donnee) 
%       sinusoide(donnee, [V1 V2])
%       sinusoide(donnee, PROF)
%       sinusoide(donnee,PROF,V)
%       sinusoide(donnee,PROF,orientation)
% @ Autres fichiers utiles :
%   sinusoidefig.m     =|=> figure
%   sinusoidefig.fig   =|
%
% Aide modifiee le 05/08/01
% DOAN Mai Linh - IPGP

function sinusoide(donnee,PROF,V,orientation)
if nargin==0
   disp('Aucune donnee n''a ete rentree');
   return
end   
if ~ischar(donnee)
    %Gestion des paramtre
    % accentuation des contrastes :
    switch nargin
    case 1
        data=donnee;
        prof=1:size(donnee,1);
        orientation=[];
    case 2
        if length(PROF)==2
            V=PROF;
            data=donnee(V(1):V(2),:);
            prof=V(1):V(2);
            orientation=[];
        else
            data=donnee;
            prof=PROF;
            orientation=[];
        end
    case 3;
        data=donnee(V(1):V(2),:);
        prof=PROF(V(1):V(2));
        orientation=[];
    case 4
        data=donnee(V(1):V(2),:);
        prof=PROF(V(1):V(2));       
    otherwise
        return
    end
    
    % preparation de la figure de plot
    fig=sinusoidefig;
    set(gcf,'Name','sinusoide');
   % colormenu;
    uimenu('label','vue3D','Callback','sinusoide(''viewforage'')','ForegroundColor',[1 0 0])
    %set(fig,'Renderer','zbuffer' )
    load bramanscolor;
    colormap(map);
    % Plot de l'image de fond
    % accentuation des contrastes :
    % L'image est mise "tete en haut"
%ff    [data,prof]=interprof(data,prof); % Interpolation maison et assemblage en ordre croissant
    nbelectrodes=size(data,2);
    axes(findobj(fig,'Tag','image'));
    set(gca,'Xlim',[0.5 nbelectrodes+0.5],'Ylim', [min(prof) max(prof)])
    imagesc(1:size(data,2),prof,data);
    hold on;
    xlim=get(gca,'Xlim');
    ylim=get(gca,'Ylim');
    h1=line([xlim(1) xlim(2)],...
        [(ylim(1)+ylim(2))/2 (ylim(1)+ylim(2))/2],...
        'Color',[0 0 0],...
        'Tag','sinus',...
        'ButtonDownFcn','sinusoide(''selection'')',...
        'LineWidth',2,...
        'EraseMode','normal');
    if ~isempty(orientation)
        if ndims(orientation)>1
            orientation=mean(orientation,1);
        end   
        incidence=orientation(1);
        azimut=orientation(2);
        set(findobj(gcf,'Tag','incidenceforage'),'string',num2str(incidence));
        set(findobj(gcf,'Tag','azimutforage'),'string',num2str(azimut));
        setappdata(gcf,'orientation',orientation);
        setappdata(gcf,'donnee',donnee);
        setappdata(gcf,'prof',PROF);
        setappdata(gcf,'blocs',V);
    end
else
    switch donnee
    case 'selection'
        % 1e clic : selection de la pointe haut ou bas
        hndl = get(gcbf,'CurrentObject');
        % Effet graphique et initialisation
        set(hndl,'Visible','off','LineWidth', 5);
        set(hndl,'Visible','on');
        set(gcf,'WindowButtonUpFcn', 'sinusoide(''relache'');', ...
            'WindowButtonMotionFcn', 'sinusoide(''deplace'');');
        % Veux t'on modifer un extermum ou un maximum ? 
        lineaxis = get(hndl,'Parent');
        point = get(lineaxis,'CurrentPoint');
        ordonnee=point(1,2);
        ydata=get(hndl, 'Ydata');
        if ordonnee >= mean(ydata)
            set(hndl, 'Userdata',1)
        else
            set(hndl, 'Userdata',-1)
        end
    case 'deplace'
        % On deplace la souris et on repre l'objet courant (la ligne)
        hndl = get(gcbf,'CurrentObject');
        %Obtention des donnes antrieures
        xdata=get(hndl,'Xdata');
        ydata=get(hndl,'Ydata');
        %  Get the current cursor point and axis limits
        lineaxis = get(hndl,'Parent');
        point = get(lineaxis,'CurrentPoint');
        xlim  = get(lineaxis,'Xlim');
        ylim  = get(lineaxis,'Ylim');
        % On fait en sorte que le point ne sorte pas du cadre
        point=[min(max(point(1,1),xlim(1)),xlim(2));...
                min(max(point(1,2),ylim(1)),ylim(2))];
        % On determine les positions extremales
        % et on en deduit les amplitudes et offset
        ismax=get(hndl,'Userdata');
        switch ismax
        case 1   % Point extremal haut
            maxX = point(1);
            maxY = point(2);
            minY = min(ydata);
            if maxY<minY
                set(hndl,'Userdata',-1)
                sinusoide('deplace');
                return;
            end
            decalage=(maxY+minY)/2;
            amplitude = (maxY-minY)/2;
            phase = maxX;
        case -1 % Point extremal bas
            minX = point(1);
            minY = point(2);
            maxY = max(ydata);
            if maxY<minY
                set(hndl,'Userdata',1)
                sinusoide('deplace');
                return;
            end
            decalage=(maxY+minY)/2;
            amplitude = (maxY-minY)/2;
            phase = xlim(1)+mod(minX+(xlim(2)-xlim(1))/2,(xlim(2)-xlim(1)));
        otherwise 
            return
        end          
        % Determination parametrique de la courbe et trace
        newxdata=linspace(xlim(1),xlim(2),100);
        newydata=decalage+amplitude*cos((newxdata-phase)*2*pi/...
            (xlim(2)-xlim(1)));
        %  Update the slider line display
        set(hndl, 'Xdata', newxdata, 'Ydata',newydata); 
        sinusoide('orientation');
    case 'relache'
        hndl = get(gcbf,'CurrentObject');
        set(gcf, 'WindowButtonUpFcn','', 'WindowButtonMotionFcn','');
        set(hndl,'Visible','off');
        set(hndl, 'LineWidth', 2, ...
            'ButtonDownFcn', 'sinusoide(''selection'');');
        set(hndl,'Visible','on','Userdata',0);
        sinusoide('orientation');
    case 'orientation'
        % Le coeur du programme, on  y calcule azimuth et incidence
        % de la normale a la fracture 
        % 1- Si la sonde est horizontale et orientee vers le nord
        % Une figure sur la fenetre sert a prciser les notations
        % 1-a Obtention de la phase
        % 1e cas L'image est orientee selon la generatrice haute
        % L'angle de la rotation propore correspond  la phse du minimum
        hndl = get(gcbf,'CurrentObject');
        % Obtention des donnes antrieures
        xdata=get(hndl,'Xdata');
        ydata=get(hndl,'Ydata');
        [maxY,imax]=max(ydata);
        [minY,imin]=min(ydata);
        % On obtient la rotation autour de la verticale (fleche verte)
        % On prend le max et non le min comme le suggre la figure
        % car en mode image l'axe vertical est  l'envers !
        rotation=imin/length(xdata)*2*pi  ; % en radian
        % Il faut connaitre le diametre pour le basculement (flche bleue)
        %Obtention du diamtre donn en cm . On le convertit en m 
        diametre=str2num(get(findobj(gcf,'tag','diametre'),'string'))/100; 
        basculement=atan2(maxY-minY,diametre); 
        % Insciption graphique
        set(findobj(gcbf,'Tag','rotation'),'string',num2str(rotation*180/pi));
        set(findobj(gcbf,'Tag','basculement'),...
            'string',num2str(basculement*180/pi));
        % Passage au referentiel absolu si les donnees le permettent
        if isappdata(gcbf,'orientation')
            orientation=getappdata(gcbf,'orientation');
            lambda=orientation(1);  % incidence du forage
            gamma=orientation(2);   % azimut du forage
            % matrice du passage pour passser de la base orthonormee
            % (generatrice haute duforage, direction perpendiculaire,
            % axe du forage dirige vers les profondeurs ) 
            % -> (nord horizontal, est horizontal, profondeur)
            passage=[cos(gamma)   sin(gamma) 0;...
                    -sin(gamma)    cos(gamma) 0;...
                    0             0          1]...
                *...
                [cos(lambda)  0 sin(lambda);...	
                    0            1  0          ;...
                    -sin(lambda) 0 cos(lambda)];
            % Dans la base
            % (generatrice haute duforage, direction
            % perpendiculaire, axe du forage dirige vers les
            % profondeurs ) 
            % le vecteur a pour coordonnees
            %(cos(rotation)*sin(basculement),
            %sin(rotation)*sin(basculement), -cos(basculement))  
            % D'ou l'expression de ce vecteur dans la base
            % (nord horizontal, est horizontal, profondeur):
            vecteur=passage*[cos(rotation)*sin(basculement);...
                    sin(rotation)*sin(basculement);...
                    -cos(basculement)];
            %normalisation
            vecteur=vecteur/(norm(vecteur)+eps);
            % D'apres les schemas de la figure,
            % Zup.n=|Zup|*|n|*cos(incidence)
            % Zup=[0 0 -1] dans (nord horizontal, est horizontal, profondeur)
            incidence=acos(dot([0 0 -1],vecteur))*180/pi;
            azimut=acos(dot([1 0 0],vecteur))*180/pi;
            set(findobj(gcbf,'Tag','incidence'),'string',num2str(incidence));
            set(findobj(gcbf,'Tag','azimut'),'string',num2str(azimut));
        end
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        %% Boutoon 'sinusoide visible'
    case 'visible'
        set(findobj(gcbf,'tag','sinus'),'Visible','on');
        set(gco,'Callback','sinusoide(''invisible'')','Value',1);
    case 'invisible'
        set(findobj(gcbf,'tag','sinus'),'Visible','off');
        set(gco,'Callback','sinusoide(''visible'')','value',0);
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        %% Apparition de la lgende de la forage
    case 'legendeforage'
        set(findobj(gcbf,'tag','axeforagehaut'),'visible','on');
        set(findobj(gcbf,'tag','axeforagehorizontal'),'visible','on');
        pause(2);
        set(findobj(gcbf,'tag','axeforagehaut'),'visible','off');
        set(findobj(gcbf,'tag','axeforagehorizontal'),'visible','off');
    case 'viewforage'
        sinus=findobj(gcbf,'Tag','sinus');
        xdata=get(sinus,'Xdata');
        ydata=get(sinus,'Ydata');
        [maxY,imax]=max(ydata);
        [minY,imin]=min(ydata);
        diametre=str2num(get(findobj(gcf,'tag','diametre'),'string'))/100; % En m
        basculement=atan2(maxY-minY,diametre); 
        rotation=imax/length(xdata)*2*pi  ; % en radian
        position=mean(ydata);
        donnee=getappdata(gcbf,'donnee');
        prof=getappdata(gcbf,'prof');
        V=getappdata(gcbf,'blocs');
        V=sort(V);
        fractures=[position rotation basculement];
        if isempty(V)
            V=[1 length(prof)];
        end
        blocelectrodes=find(abs(prof-(position+3))<0.01); 
        % Les electrodes sont 3m plus bas
        if ~isempty(blocelectrodes);
            if blocelectrodes < V(1)
                V=[blocelectrodes V(2)];
            end
            if blocelectrodes > V(2)
                V=[V(1) blocelectrodes];
            end
        end
        fig=viewforage(donnee,prof,V,fractures,diametre);
        figure(fig)
    end
end